--[[--------------------------------------------------------------
Loot Spec Icon
by Sunsequent

Small icon on player portrait showing loot specialization.
Left click changes loot spec to the next one, right click sets it
to Current Specialization.

Version 1.2.2
Release Date 03.05.2023
--------------------------------------------------------------]]--

--[[ CHANGELOG ]]--[[
Version 1.2.2
- Update for patch 10.1
- Add icon for addon

Version 1.2.1
- Move default icon position to not obscure combat indicator on player frame
 
Version 1.2.0
- Update for Dragonflight
- Adjusted default icon position to match new player frame
 
Version 1.1.3
- Adjusted default icon position on classes with pets/secondary power bar
- When unlocked, icon changes from spec icon to configuration icon
 
Version 1.1.2
- Fix for unlocked button staying visible during cutscenes
 
Version 1.1.1
- Fix for double binding header error
 
Version 1.1.0
- Added option to move, resize and hide the icon
 
Version 1.0.1
- Added Localization
]]--

--SELECT_LOOT_SPECIALIZATION = "Loot Specialization";
--ERR_LOOT_SPEC_CHANGED_S = "Loot Specialization set to: %s";
--LOOT_SPECIALIZATION_DEFAULT = "Current Specialization ( %s )";
--NEXT = "Next";
--REFORGE_CURRENT = "Current";
--LFG_TYPE_NONE = "None";

_G["BINDING_HEADER_LOOTSPECICON"] = "Loot Spec Icon";

_G["BINDING_NAME_CLICK LootSpecIconButton:LeftButton"] = NEXT.." "..SELECT_LOOT_SPECIALIZATION
_G["BINDING_NAME_CLICK LootSpecIconButton:RightButton"] = REFORGE_CURRENT.." "..SELECT_LOOT_SPECIALIZATION

SLASH_LOOTSPECICON1, SLASH_LOOTSPECICON2 = "/lootspecicon", "/lsi";

local iconLocked = true;

function SlashCmdList.LOOTSPECICON(msg, editbox)
    local _, className = UnitClass("player");
	
	if msg == "unlock" then
        if iconLocked then
			print("Loot Spec Icon unlocked, drag to move to a new position, left/right click to change size.");
			LootSpecIcon_UnlockIcon();
		else
			print("Loot Spec Icon locked.");
			LootSpecIcon_LockIcon();
		end
		
        return;
    end
	
	if msg == "hide" then
		if LootSpecIconButton:IsShown() then
			print("Loot Spec Icon hidden.");
			LootSpecIconDB[className].Hidden = true;
			LootSpecIconButton:Hide();
		else
			print("Loot Spec Icon shown.");
			LootSpecIconDB[className].Hidden = false;
			LootSpecIconButton:Show();
		end
		
		return;
	end
	
	if msg == "reset" then
		print("Loot Spec Icon reset.");
		LootSpecIcon_LockIcon();
		
		LootSpecIcon_SetDefaultPosition();
		
		LootSpecIconDB[className].Position = nil;
		LootSpecIconDB[className].Hidden = false;
		LootSpecIconButton:Show();
		
		return;
	end
	
	print("Loot Spec Icon (also /lsi) commands:");
	print("/lootspecicon unlock");
	print("/lootspecicon hide");
	print("/lootspecicon reset");
end

function LootSpecIcon_SetDefaultPosition()
	local _, className = UnitClass("player");
	
	LootSpecIconButton:ClearAllPoints();
	LootSpecIconButton:SetParent(PlayerFrame);
	LootSpecIconButton:SetFrameStrata("MEDIUM");
	
	LootSpecIconButton:SetScale(0.8);
	LootSpecIconButton:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 16, -18);
end

local lootSpecIconClicked = false;
local currentSpecializationLabel;

PlayerFrame:HookScript("OnHide", function() 
	LootSpecIconButton:Hide(); 
end);

PlayerFrame:HookScript("OnShow", function() 
	local _, className = UnitClass("player");
	if not LootSpecIconDB[className].Hidden then
		LootSpecIconButton:Show();
	end
end);

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(self, event, ...)
	if event == "ADDON_LOADED" then
		local loadedAddon = select(1, ...);
		
		if loadedAddon == "LootSpecIcon" then
			currentSpecializationLabel = (string.gsub((string.gsub(LOOT_SPECIALIZATION_DEFAULT, "%( ", "%(")), " %)", "%)"))

			if LootSpecIconDB == nil then
				LootSpecIconDB = { };
			end
			
			local _, className = UnitClass("player");
			if LootSpecIconDB[className] == nil then
				LootSpecIconDB[className] = { Hidden = false };
			end
			
			if LootSpecIconDB[className].Position ~= nil then
				LootSpecIconButton:ClearAllPoints();
				LootSpecIconButton:SetParent(nil);
				LootSpecIconButton:SetFrameStrata("MEDIUM");
				LootSpecIconButton:SetScale(LootSpecIconDB[className].Position.scale);
				LootSpecIconButton:SetPoint(LootSpecIconDB[className].Position.point, UIParent, LootSpecIconDB[className].Position.relativePoint, LootSpecIconDB[className].Position.ofsx, LootSpecIconDB[className].Position.ofsy);
			else
				LootSpecIcon_SetDefaultPosition();
			end
		end
	else
		LootSpecIcon_SetLootSpecIcon();
		
		local _, className = UnitClass("player");
		if LootSpecIconDB[className].Hidden then
			LootSpecIconButton:Hide();
		else
			LootSpecIconButton:Show();
		end
		
		if lootSpecIconClicked and GameTooltip:IsShown() and GameTooltip:GetOwner() == LootSpecIconButton then
			LootSpecIcon_ShowLootSpecializationTooltip();
		end
		
		lootSpecIconClicked = false;
	end
end)
f:RegisterEvent("ADDON_LOADED");
f:RegisterEvent("PLAYER_ENTERING_WORLD");
f:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
f:RegisterEvent("PLAYER_LOOT_SPEC_UPDATED");

function LootSpecIcon_SetLootSpecIcon()
	local _, className, classID = UnitClass("player");
	local numClassSpecializations = GetNumSpecializationsForClassID(classID);
	
	local icon = 134400;
	local texCoords = { 0, 0, 0, 1, 1, 0, 1, 1 };
	local currentLootSpec = GetLootSpecialization();
	if currentLootSpec == 0 then
		local currentSpec = GetSpecialization();
		if currentSpec then
			if currentSpec <= numClassSpecializations then
				icon = select(4, GetSpecializationInfo(currentSpec));
			else
				icon = "Interface\\TargetingFrame\\UI-Classes-Circles";
				texCoords = CLASS_ICON_TCOORDS[strupper(className)];
			end
		end		
	else
		icon = select(4, GetSpecializationInfoByID(currentLootSpec));
	end
	
	LootSpecIconBackground:SetTexture(icon);
	LootSpecIconBackground:SetTexCoord(unpack(texCoords));
	LootSpecIconBackground:SetDesaturated(nil);
	LootSpecIconBackground:SetVertexColor(255/255, 255/255, 255/255);
end

function LootSpecIcon_SetNextLootSpecialization(self, button)
	lootSpecIconClicked = true;
	local localizedClass, className, classID = UnitClass("player");
	local numClassSpecializations = GetNumSpecializationsForClassID(classID);
	
	if button=="LeftButton" then
		if iconLocked then
			local currentLootSpecID = GetLootSpecialization();
			local currentLootSpecNum;
			local specIDToSet;
			
			if currentLootSpecID == 0 then
				specIDToSet = GetSpecializationInfoForClassID(classID, 1);		
			else
				for specNum = 1, numClassSpecializations do
					local specID = GetSpecializationInfoForClassID(classID, specNum);
					
					if currentLootSpecID == specID then
						currentLootSpecNum = specNum;
						break;
					end
				end
				
				if currentLootSpecNum == numClassSpecializations then
					specIDToSet = GetSpecializationInfoForClassID(classID, 1);			
				else
					specIDToSet = GetSpecializationInfoForClassID(classID, currentLootSpecNum + 1);			
				end
			end
			
			SetLootSpecialization(specIDToSet);
		else
			if LootSpecIconDB[className].Position.scale < 1.5 then
				LootSpecIconDB[className].Position.scale = LootSpecIconDB[className].Position.scale + 0.1;
				LootSpecIconButton:SetScale(LootSpecIconDB[className].Position.scale);
			end
		end
	elseif button=="RightButton" then
		if iconLocked then
			local currentSpec = GetSpecialization();
			local currentSpecName = LFG_TYPE_NONE;
			if currentSpec then			
				if currentSpec <= numClassSpecializations then
					currentSpecName = select(2, GetSpecializationInfo(currentSpec));
				else
					currentSpecName = localizedClass;
				end
			end
			
			DEFAULT_CHAT_FRAME:AddMessage((ERR_LOOT_SPEC_CHANGED_S):format((currentSpecializationLabel):format(currentSpecName)), 255/255, 255/255, 0/255);
			SetLootSpecialization(0);
		else
			if LootSpecIconDB[className].Position.scale > 0.5 then
				LootSpecIconDB[className].Position.scale = LootSpecIconDB[className].Position.scale - 0.1;
				LootSpecIconButton:SetScale(LootSpecIconDB[className].Position.scale);
			end
		end
	end
end

function LootSpecIcon_ShowLootSpecializationTooltip()
	if iconLocked then
		local currentLootSpec = GetLootSpecialization();
		local _, name = GetSpecializationInfoByID(currentLootSpec);
		local localizedClass, className, classID = UnitClass("player");
		local numClassSpecializations = GetNumSpecializationsForClassID(classID);
		
		GameTooltip:Hide();
		GameTooltip:SetOwner(LootSpecIconButton, "ANCHOR_RIGHT");
		
		if name then
			GameTooltip:AddLine(SELECT_LOOT_SPECIALIZATION..": "..name);
		else
			local currentSpec = GetSpecialization();
			local currentSpecName = LFG_TYPE_NONE;
			if currentSpec then
				if currentSpec <= numClassSpecializations then
					currentSpecName = select(2, GetSpecializationInfo(currentSpec));
				else
					currentSpecName = localizedClass;
				end
			end
			
			GameTooltip:AddLine(SELECT_LOOT_SPECIALIZATION..": "..(currentSpecializationLabel):format(currentSpecName));
		end
		
		GameTooltip:Show();
	else
		GameTooltip:Hide();
		GameTooltip:SetOwner(LootSpecIconButton, "ANCHOR_RIGHT");
		GameTooltip:AddLine("Loot Spec Icon unlocked");		
		GameTooltip:Show();
	end
end

function LootSpecIcon_UnlockIcon()
	iconLocked = false;
	local _, className = UnitClass("player");
	LootSpecIconButton:Show();
	LootSpecIconDB[className].Hidden = false;
	LootSpecIconButton:RegisterForDrag("LeftButton");
	LootSpecIconButton:SetMovable(true);
	
	local texCoords = { 0, 0, 0, 1, 1, 0, 1, 1 };
	LootSpecIconBackground:SetTexture(136243);
	LootSpecIconBackground:SetTexCoord(unpack(texCoords));
	LootSpecIconBackground:SetDesaturated(1);
	LootSpecIconBackground:SetVertexColor(255/255, 0/255, 0/255);
	
	if LootSpecIconButton:GetParent() == PlayerFrame then
		LootSpecIconButton:ClearAllPoints();
		LootSpecIconButton:SetParent(nil);
		LootSpecIconButton:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
		LootSpecIconButton:SetFrameStrata("MEDIUM");
		LootSpecIconButton:SetScale(0.8);
		
		if LootSpecIconDB[className].Position == nil then
			LootSpecIconDB[className].Position = { };
		end
		LootSpecIconDB[className].Position.scale = 0.8;
	end
end

function LootSpecIcon_LockIcon()
	iconLocked = true;
	LootSpecIconButton:RegisterForDrag();
	LootSpecIconButton:SetMovable(false);
	LootSpecIcon_SetLootSpecIcon();
end

function LootSpecIcon_DragStart(self, button)
	self:LockHighlight();
	self:StartMoving();
	self.isMoving = true;
end

function LootSpecIcon_DragStop(self, button)
	self:UnlockHighlight();
	self:StopMovingOrSizing();
	self:SetUserPlaced(false);
	self.isMoving = false;
	
	local point, relativeTo, relativePoint, ofsx, ofsy = self:GetPoint();
	
	local _, className = UnitClass("player");
	if LootSpecIconDB[className].Position == nil then
		LootSpecIconDB[className].Position = { };
	end
	
	LootSpecIconDB[className].Position.point = point;
	LootSpecIconDB[className].Position.relativePoint = relativePoint;
	LootSpecIconDB[className].Position.ofsx = ofsx;
	LootSpecIconDB[className].Position.ofsy = ofsy;
end
